/*  main.c
 *
 *  Main body of xwxapt application
 */

/*
 *  xwxapt: An application to decode APT signals from
 *  weather satellites and produce image(s) of the weather.
 *
 *  Copyright (C) 2001  Neoklis Kyriazis
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details:
 *
 *  http://www.gnu.org/copyleft/gpl.txt
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include "main.h"


GtkWidget
  *xwxapt_main_window, /* xwxapt's top window   */
  *gbl_text_scroller,  /* Text view scroller    */
  *gbl_level_pbar,     /* Audio level prog. bar */
  *gbl_sync_pbar,      /* Sync level prog. bar  */
  *gbl_status_table,   /* Status widgets' table */
  *gbl_sync_icon,      /* Sync status indicator */
  *gbl_apt_image;      /* Image to be displayed */

/* Text buffer for text view */
GtkTextBuffer *gbl_text_buffer;

/* Pixel buffer for display */
GdkPixbuf *gbl_pixbuf = NULL;

/* Pointer to pixbuf pixels */
guchar *gbl_pixel_buf;

/* Pixbuf rowstride and num of channels */
gint
  gbl_rowstride,
  gbl_n_channels;

/* Satellite type names */
char *gbl_sat_names[] = { SAT_NAMES };

/* List of required sync position references */
int gbl_sync_refs[] = { SYNC_REFS };

int
  gbl_sat_type,   /* Weather satellite type number   */
  gbl_duration;   /* Time limit in sec on processing */

char
  gbl_samples_file[MAX_FILE_NAME+1], /* Samples file name */
  gbl_image_file  [MAX_FILE_NAME+1]; /* Image files name  */

/* Pointer to samples file */
FILE *gbl_samples_fp = NULL;

/* Signal handler */
static void sig_handler( int signal );

/* Runtime config data */
rc_data_t rc_data;

/*------------------------------------------------------------------------*/

  int
main (int argc, char *argv[])
{
  /* Command line option returned by getopt() */
  int option;

  /* getopt() variables */
  extern char *optarg;
  extern int optind, opterr, optopt;

  /* New and old actions for sigaction() */
  struct sigaction sa_new, sa_old;


  /* Initialize new actions */
  sa_new.sa_handler = sig_handler;
  sigemptyset( &sa_new.sa_mask );
  sa_new.sa_flags = 0;

  /* Register function to handle signals */
  sigaction( SIGINT,  &sa_new, &sa_old );
  sigaction( SIGSEGV, &sa_new, 0 );
  sigaction( SIGFPE,  &sa_new, 0 );
  sigaction( SIGTERM, &sa_new, 0 );
  sigaction( SIGABRT, &sa_new, 0 );

  /* Process command line options */
  while( (option = getopt(argc, argv, "hv") ) != -1 )
	switch( option )
	{
	  case 'h' : /* Print usage and exit */
		Usage();
		return(0);

	  case 'v' : /* Print version */
		fprintf( stderr, "%s %s\n", PACKAGE, VERSION );
		return(0);

	  default: /* Print usage and exit */
		Usage();
		exit(-1);

	} /* End of switch( option ) */

  /* Start GTK+ */
  gtk_set_locale();
  gtk_init( &argc, &argv );

  /* Defaults/initialization */
  rc_data.default_dur = gbl_duration = 600;
  gbl_sat_type        = NOAA;
  gbl_image_file[0]   = '\0';
  gbl_samples_file[0] = '\0';
  SetFlag( IMAGE_NORMALIZE | IMAGE_COLORIZE );

  /* Create xwxapt window */
  xwxapt_main_window = create_main_window();

  /* Create global sync status widgets' table */
  gbl_status_table = lookup_widget( xwxapt_main_window, "status_frame" );

  /* Create global sync status indicator icon */
  gbl_sync_icon = lookup_widget( xwxapt_main_window, "sync_ind" );

  /* Create the text view scroller */
  gbl_text_scroller = lookup_widget( xwxapt_main_window, "text_scrolledwindow" );

  /* Get text buffer */
  gbl_text_buffer = gtk_text_view_get_buffer
	( GTK_TEXT_VIEW(lookup_widget(xwxapt_main_window, "textview")) );

  /* Create some rendering tags */
  gtk_text_buffer_create_tag( gbl_text_buffer, "black",
	  "foreground", "black", NULL);
  gtk_text_buffer_create_tag( gbl_text_buffer, "red",
	  "foreground", "red", NULL);
  gtk_text_buffer_create_tag( gbl_text_buffer, "orange",
	  "foreground", "orange", NULL);
  gtk_text_buffer_create_tag( gbl_text_buffer, "green",
	  "foreground", "darkgreen", NULL);
  gtk_text_buffer_create_tag( gbl_text_buffer, "bold",
	  "weight", PANGO_WEIGHT_BOLD, NULL);

  /* Create the progress bars */
  gbl_level_pbar = lookup_widget( xwxapt_main_window, "audio_lev" );
  gbl_sync_pbar  = lookup_widget( xwxapt_main_window, "sync_lev" );

  /* Open xwxapt window */
  gtk_widget_show( xwxapt_main_window );
  Initialize_Top_Window();
  Show_Message( "Welcome to xwxapt version "VERSION, "bold" );

  g_idle_add( Load_Config, NULL );

  gtk_main ();

  return 0;
}

/*------------------------------------------------------------------------*/

/*  Initialize_Top_Window()
 *
 *  Initializes xwxapt's top window
 */

  void
Initialize_Top_Window( void )
{
  /* The scrolled window image container */
  GtkWidget *image_scroller;

  if( gbl_pixbuf != NULL )
  {
	g_object_unref( gbl_pixbuf );
	gbl_pixbuf = NULL;
  }

  switch( gbl_sat_type )
  {
	case NOAA: /* Create pixbuff for NOAA images */
	  gbl_pixbuf = gdk_pixbuf_new( GDK_COLORSPACE_RGB, FALSE, 8,
		  NOAA_IMAGE_WIDTH, gbl_duration );
	  break;

	case METEOR: /* Create pixbuff for METEOR images */
	  gbl_pixbuf = gdk_pixbuf_new( GDK_COLORSPACE_RGB, FALSE, 8,
		  METEOR_IMAGE_WIDTH/2, gbl_duration );
  }

  /* Error, not enough memory */
  if( gbl_pixbuf == NULL)
  {
	Show_Message( "Failed to allocated memory to pixbuf - Quit", "red" );
	Error_Dialog();
	return;
  }

  /* Fill pixbuf with background color */
  gdk_pixbuf_fill( gbl_pixbuf, 0xaaaaaaff );

  /* Get details of pixbuf */
  gbl_pixel_buf  = gdk_pixbuf_get_pixels(     gbl_pixbuf );
  gbl_rowstride  = gdk_pixbuf_get_rowstride(  gbl_pixbuf );
  gbl_n_channels = gdk_pixbuf_get_n_channels( gbl_pixbuf );

  /* Globalize image to be displayed */
  gbl_apt_image = lookup_widget( xwxapt_main_window, "image" );

  /* Set apt image from pixbuff */
  gtk_image_set_from_pixbuf( GTK_IMAGE(gbl_apt_image), gbl_pixbuf );
  gtk_widget_show( gbl_apt_image );

  /* Set window size as required */
  image_scroller = lookup_widget( xwxapt_main_window, "image_scrolledwindow" );
  gtk_widget_set_size_request( image_scroller,
	  (gbl_duration <= rc_data.default_dur ?
	   NOAA_IMAGE_WIDTH + 8 : NOAA_IMAGE_WIDTH + 26), /* Set width */
	  (gbl_duration <= rc_data.default_dur ?
	   gbl_duration + 8 : rc_data.default_dur + 8) ); /* Set height */

  gtk_window_resize( GTK_WINDOW(xwxapt_main_window), 10, 10 );

} /* Initialize_Top_Window() */

/*------------------------------------------------------------------------*/

/*  sig_handler()
 *
 *  Signal Action Handler function
 */

static void sig_handler( int signal )
{
  Cleanup();

  fprintf( stderr, "\n" );
  switch( signal )
  {
	case SIGINT :
	  fprintf( stderr, "%s\n", "xwxapt: Exiting via User Interrupt" );
	  exit(-1);
	  break;

	case SIGSEGV :
	  fprintf( stderr, "%s\n", "xwxapt: Segmentation Fault" );
	  exit(-1);
	  break;

	case SIGFPE :
	  fprintf( stderr, "%s\n", "xwxapt: Floating Point Exception" );
	  exit(-1);
	  break;

	case SIGABRT :
	  fprintf( stderr, "%s\n", "xwxapt: Abort Signal received" );
	  exit(-1);
	  break;

	case SIGTERM :
	  fprintf( stderr, "%s\n", "xwxapt: Termination Request received" );
	  exit(-1);
  }

} /* End of sig_handler() */

/*------------------------------------------------------------------------*/

